var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
import { A as Action$1, a as Store$1 } from "./storePersist.js";
export { S as StorePersist } from "./storePersist.js";
import React, { useRef, useState, useEffect, useMemo, createContext, useContext } from "react";
import { circularDeepEqual } from "fast-equals";
import "immer";
function useEqualityRef(x) {
  const ref = useRef(x);
  if (!circularDeepEqual(x, ref.current)) {
    ref.current = x;
  }
  return ref.current;
}
const ignore = () => {
};
function useAction(action, arg, { watchOnly, updateOnMount, clearBeforeUpdate, dormant, throttle } = {}) {
  const [counter, setCounter] = useState(0);
  useEffect(() => {
    if (updateOnMount && !dormant)
      action.execute(arg, { clearBeforeUpdate }).catch(ignore);
  }, []);
  useEffect(() => {
    if (dormant) {
      return;
    }
    if (!watchOnly)
      action.get(arg).catch(ignore);
    return action.subscribe(arg, () => {
      setCounter((c) => c + 1);
      if (!watchOnly)
        action.get(arg).catch(ignore);
    }, { runNow: true, throttle });
  }, [action, useEqualityRef(arg), watchOnly, dormant, throttle]);
  return useMemo(() => {
    var _a, _b;
    if (dormant) {
      return [void 0, { error: void 0, isLoading: false }];
    }
    const instance = action.getCache(arg);
    return [
      ((_a = instance == null ? void 0 : instance.current) == null ? void 0 : _a.kind) === "value" ? instance.current.value : void 0,
      { error: ((_b = instance == null ? void 0 : instance.current) == null ? void 0 : _b.kind) === "error" ? instance.current.error : void 0, isLoading: !!(instance == null ? void 0 : instance.inProgress) }
    ];
  }, [action, useEqualityRef(arg), dormant, counter]);
}
class Action extends Action$1 {
  useAction(arg, options) {
    return useAction(this, arg, options);
  }
}
function createSelector(stringSelector) {
  const parts = stringSelector.split(".");
  return (state) => parts.reduce((v, p) => v instanceof Object && p in v ? v[p] : void 0, state);
}
function setWithSelector(state, stringSelector, value) {
  const parts = stringSelector.split(".");
  const prefix = parts.slice(0, -1);
  const last = parts[parts.length - 1];
  for (const part of prefix) {
    if (state instanceof Object && part in state) {
      state = state[part];
    } else {
      throw Error(`Could not set ${stringSelector} because path ${stringSelector} doesn't exist`);
    }
  }
  if (state instanceof Object) {
    state[last] = value;
  } else {
    throw Error(`Could not set ${stringSelector} because path ${stringSelector} doesn't exist`);
  }
}
function useStoreState(store, ...args) {
  var _a, _b, _c, _d;
  let selector, deps, options;
  if (args[0] instanceof Function) {
    selector = args[0];
    deps = (_a = args[1]) != null ? _a : [];
    options = (_b = args[2]) != null ? _b : {};
  } else if (typeof args[0] === "string") {
    selector = createSelector(args[0]);
    deps = [args[0]];
    options = (_c = args[1]) != null ? _c : {};
  } else {
    selector = (x) => x;
    deps = [];
    options = (_d = args[0]) != null ? _d : {};
  }
  const [counter, setCounter] = useState(0);
  const value = useMemo(() => selector(store.getState()), [store, counter, ...deps]);
  useEffect(() => store.subscribe(selector, () => setCounter((c) => c + 1), { runNow: true, throttle: options.throttle }), [store, ...deps]);
  return value;
}
function useStoreProp(store, selector) {
  const value = useStoreState(store, selector);
  const update = (value2) => store.update((state) => setWithSelector(state, selector, value2));
  return [value, update];
}
class Store extends Store$1 {
  useState(...args) {
    return useStoreState(this, ...args);
  }
  useProp(selector) {
    return useStoreProp(this, selector);
  }
}
class StoreScope {
  constructor(defaultValue) {
    this.defaultValue = defaultValue;
    this.Provider = ({ children, store }) => {
      const _store = useMemo(() => store != null ? store : new Store(this.defaultValue), [store]);
      return /* @__PURE__ */ React.createElement(this.context.Provider, {
        value: _store
      }, children);
    };
    this.withScope = (Component) => {
      return (args) => /* @__PURE__ */ React.createElement(this.Provider, null, /* @__PURE__ */ React.createElement(Component, __spreadValues({}, args)));
    };
    this.context = createContext(new Store(defaultValue));
  }
  useStore() {
    return useContext(this.context);
  }
  useState(...args) {
    const store = this.useStore();
    return useStoreState(store, ...args);
  }
  useProp(selector) {
    const store = this.useStore();
    return useStoreProp(store, selector);
  }
}
export { Action, Store, StoreScope, useAction, useStoreState };
//# sourceMappingURL=react.js.map
